@@ -1,6 +1,8 @@
+use proc_macro::Diagnostic;
 use rust_ad_core::traits::*;
 use rust_ad_core::*;
 use std::collections::HashMap;
+use syn::spanned::Spanned;
 
 pub fn update_forward_return(s: Option<&mut syn::Stmt>, function_inputs: &[String]) {
     *s.unwrap() = match s {
@@ -34,11 +36,11 @@ pub fn update_forward_return(s: Option<&mut syn::Stmt>, function_inputs: &[Strin
 }
 
 /// Insperses values with respect to the preceding values.
-pub fn interspese_succedding_stmts<K>(
+pub fn interspese_succedding_stmts<K, R>(
     x: Vec<syn::Stmt>,
     extra: K,
-    f: fn(&syn::Stmt, &K) -> Option<syn::Stmt>,
-) -> Vec<syn::Stmt> {
+    f: fn(&syn::Stmt, &K) -> Result<Option<syn::Stmt>, R>,
+) -> Result<Vec<syn::Stmt>, R> {
     let len = x.len();
     let new_len = len * 2 - 1;
     let mut y = Vec::with_capacity(new_len);
@@ -47,7 +49,12 @@ pub fn interspese_succedding_stmts<K>(
         y.push(last);
     }
     for a in x_iter {
-        if let Some(b) = f(&a, &extra) {
+        let res = f(&a, &extra);
+        let opt = match res {
+            Ok(r) => r,
+            Err(e) => return Err(e),
+        };
+        if let Some(b) = opt {
             // for c in crate::unwrap_statement(&b).into_iter() {
             //     y.push(c);
             // }
@@ -55,14 +62,14 @@ pub fn interspese_succedding_stmts<K>(
         }
         y.push(a);
     }
-    y.into_iter().rev().collect()
+    Ok(y.into_iter().rev().collect())
 }
 
 // TODO Reduce code duplication between `reverse_derivative` and `forward_derivative`
 pub fn forward_derivative(
     stmt: &syn::Stmt,
     (type_map, function_inputs): &(&HashMap<String, String>, &[String]),
-) -> Option<syn::Stmt> {
+) -> Result<Option<syn::Stmt>, ()> {
     if let syn::Stmt::Local(local) = stmt {
         if let Some(init) = &local.init {
             // eprintln!("init: {:#?}",init);
@@ -70,30 +77,60 @@ pub fn forward_derivative(
                 // Creates operation signature struct
                 let operation_sig = operation_signature(bin_expr, type_map);
                 // Looks up operation with the given lhs type and rhs type and BinOp.
-                let operation_out_signature = SUPPORTED_OPERATIONS
-                    .get(&operation_sig)
-                    .expect("forward_derivative: unsupported operation");
+                let operation_out_signature = match SUPPORTED_OPERATIONS.get(&operation_sig) {
+                    Some(sig) => sig,
+                    None => {
+                        let error = format!("unsupported derivative for {}", operation_sig);
+                        Diagnostic::spanned(
+                            bin_expr.span().unwrap(),
+                            proc_macro::Level::Error,
+                            error,
+                        )
+                        .emit();
+                        return Err(());
+                    }
+                };
                 // Applies the forward deriative function for the found operation.
                 let new_stmt = (operation_out_signature.forward_derivative)(&stmt, function_inputs);
-                return Some(new_stmt);
+                return Ok(Some(new_stmt));
             } else if let syn::Expr::Call(call_expr) = &*init.1 {
                 // Create function in signature
                 let function_in_signature = function_signature(call_expr, type_map);
                 // Gets function out signature
-                let function_out_signature = SUPPORTED_FUNCTIONS
-                    .get(&function_in_signature)
-                    .expect("forward_derivative: unsupported function");
+                let function_out_signature = match SUPPORTED_FUNCTIONS.get(&function_in_signature) {
+                    Some(sig) => sig,
+                    None => {
+                        let error = format!("unsupported derivative for {}", function_in_signature);
+                        Diagnostic::spanned(
+                            call_expr.span().unwrap(),
+                            proc_macro::Level::Error,
+                            error,
+                        )
+                        .emit();
+                        return Err(());
+                    }
+                };
                 // Gets new stmt
                 let new_stmt = (function_out_signature.forward_derivative)(&stmt, function_inputs);
 
-                return Some(new_stmt);
+                return Ok(Some(new_stmt));
             } else if let syn::Expr::MethodCall(method_expr) = &*init.1 {
                 let method_sig = method_signature(method_expr, type_map);
-                let method_out = SUPPORTED_METHODS
-                    .get(&method_sig)
-                    .expect("forward_derivative: unsupported method");
+                let method_out = match SUPPORTED_METHODS.get(&method_sig) {
+                    Some(sig) => sig,
+                    None => {
+                        let error = format!("unsupported derivative for {}", method_sig);
+                        Diagnostic::spanned(
+                            method_expr.span().unwrap(),
+                            proc_macro::Level::Error,
+                            error,
+                        )
+                        .emit();
+                        return Err(());
+                    }
+                };
                 let new_stmt = (method_out.forward_derivative)(&stmt, function_inputs);
-                return Some(new_stmt);
+                return Ok(Some(new_stmt));
             } else if let syn::Expr::Path(expr_path) = &*init.1 {
                 // Given `let x = y;`
 
@@ -129,7 +166,7 @@ pub fn forward_derivative(
                 let new_stmt: syn::Stmt =
                     syn::parse_str(&stmt_str).expect("forward_derivative: parse fail");
 
-                return Some(new_stmt);
+                return Ok(Some(new_stmt));
             } else if let syn::Expr::Lit(expr_lit) = &*init.1 {
                 // Given `let x = y;`
 
@@ -161,9 +198,9 @@ pub fn forward_derivative(
                 let new_stmt: syn::Stmt =
                     syn::parse_str(&stmt_str).expect("forward_derivative: parse fail");
 
-                return Some(new_stmt);
+                return Ok(Some(new_stmt));
             }
         }
     }
-    None
+    Ok(None)
 }
